App.controller('Controller', function ($scope, $location, $window) {
    var _this = this;
    _this.CurrentVersion = APP_VERSION;

    _this.SelectedAuditoriumNumber = null;
    _this.SelectedAuditorium = undefined;
    _this.Auditoriums = [];
    _this.TemperatureUnits = "F";
    _this.TemperatureDecimalPlaces = 1;
    _this.PageNumber = 1;


    // watch for changes to our $location.url() object.
    $scope.$watch(function () {
        if (!_this.SelectedAuditorium && -1 != _this.SelectedAuditoriumNumber) {
            var auditorium = _this.Auditoriums[_this.SelectedAuditoriumNumber - 1];
            if (auditorium) {
                _this.SelectedAuditorium = auditorium;
                $scope.$apply();
            }
        }

        return $location.url();
    }, function (url) {
        var pageNumber = $location.search().page;
        if (pageNumber) _this.setPage(pageNumber);

        var auditoriumNumber = $location.search().auditoriumNumber;
        if (auditoriumNumber) {
            _this.setAuditorium(auditoriumNumber);
        } else {
            _this.setAuditorium(null);
        }
    });



    _this.setPage = function (pageNumber) {
        _this.PageNumber = pageNumber;
        $location.search('page', pageNumber);
    };



    _this.setAuditorium = function (auditoriumNumber) {
        // if (!_this.Auditoriums[auditoriumNumber - 1]) {
        //     auditoriumNumber = null;
        // }

        _this.SelectedAuditoriumNumber = auditoriumNumber;
        if (null != auditoriumNumber) {
            _this.SelectedAuditorium = _this.Auditoriums[auditoriumNumber - 1];
            $location.search('auditoriumNumber', auditoriumNumber);
        } else {
            _this.SelectedAuditorium = null;
            $location.search('auditoriumNumber', null);
            $location.url();
        }
    };



    // onLoggedIn callback.  this is where we will read the config file
    jnrWebsocket.addOnLoggedInListener(function onLoggedIn() {
        console.log('onLoggedIn');

        // here we get the peers.  we will use the peers to connect to the other 
        // auditoriums.  it is likely that all or most of the jniors in a ibooth
        // complex are meant for this application.
        jnrWebsocket.getRegistryListing('peers', function (node, keys) {
            jnrWebsocket.readRegistryKeys(keys, function (keyName, value) {
                if (null != value) {
                    var ipAddress = value.split(' ')[0];
                    var series3 = -1 != keyName.indexOf('$1')
                        || -1 != keyName.indexOf('$2')
                        || -1 != keyName.indexOf('$3')
                        || -1 != keyName.indexOf('$4');
                    // ignore blank ip addresses or series 3 jniors
                    if ('0.0.0.0' != ipAddress && !series3) {
                        createPeerConnection(ipAddress);
                    }
                }
            });
        });

        jnrWebsocket.readRegistryKey('appdata/ibooth/tmsipaddress', function (keyName, value) {
            if (value && null != value) {
                _this.TmsIpAddress = value;
            }
        });

        jnrWebsocket.readRegistryKey('appdata/ibooth/tmsport', function (keyName, value) {
            if (value && null != value) {
                _this.TmsPort = value;
            }
        });

        jnrWebsocket.readRegistryKey('appdata/ibooth/temperatureunits', function (keyName, value) {
            if (value && null != value) {
                _this.TemperatureUnits = value;
                $scope.$apply();
            }
        });

        jnrWebsocket.readRegistryKey('appdata/ibooth/temperaturedecimalplaces', function (keyName, value) {
            if (value && null != value) {
                _this.TemperatureDecimalPlaces = value;
                $scope.$apply();
            }
        });

        peerConnections.push(new PeerConnection(jnrWebsocket, _this.Auditoriums));
    });



    _this.getTemperature = function (auditorium) {
        if (null != auditorium && auditorium && auditorium.Data &&
            auditorium.Data.Temperature && !isNaN(auditorium.Data.Temperature)) {
            if (_this.TemperatureUnits &&
                ('C' == _this.TemperatureUnits || 'c' == _this.TemperatureUnits)) {
                var celsius = (auditorium.Data.Temperature - 32) * 5 / 9.0;
                return celsius.toFixed(_this.TemperatureDecimalPlaces) + 'C';
            } else {
                return auditorium.Data.Temperature.toFixed(_this.TemperatureDecimalPlaces) + 'F';
            }
        }
        return '';
    };



    _this.getHumidity = function (auditorium) {
        if (null != auditorium && auditorium && auditorium.Data &&
            auditorium.Data.Humidity && !isNaN(auditorium.Data.Humidity)) {
            return auditorium.Data.Humidity.toFixed(0) + '%';
        }
        return '';
    };



    _this.getTempHumidity = function (auditoriumIndex) {
        var auditorium = _this.Auditoriums[auditoriumIndex];
        if (null != auditorium && auditorium && auditorium.Data) {
            return _this.getTemperature(auditorium) + ' / ' + _this.getHumidity(auditorium);
        }
        return '';
    };



    _this.projectorTabsAuditorium = [];
    _this.launchProjectorPage = function () {
        if (null == _this.projectorTabsAuditorium[_this.SelectedAuditoriumNumber]
            || _this.projectorTabsAuditorium[_this.SelectedAuditoriumNumber].closed) {

            var url = 'http://' + _this.SelectedAuditorium.ProjectorIpAddress;
            if (_this.SelectedAuditorium.MediaBlockType.startsWith('Barco')) {
                url = 'https://' + _this.SelectedAuditorium.ProjectorIpAddress + ':43744'; ///#sms/control';
            }

            var windowObject = window.open(url, "projector-control-" + _this.SelectedAuditoriumNumber);
            _this.projectorTabsAuditorium[_this.SelectedAuditoriumNumber] = windowObject;
        } else {
            _this.projectorTabsAuditorium[_this.SelectedAuditoriumNumber].focus();
        }
    };



    _this.mediaBlockTabsAuditorium = [];
    _this.launchMediaBlockPage = function () {
        if (null == _this.mediaBlockTabsAuditorium[_this.SelectedAuditoriumNumber]
            || _this.mediaBlockTabsAuditorium[_this.SelectedAuditoriumNumber].closed) {

            var url = 'http://' + _this.SelectedAuditorium.MediaBlockIpAddress;
            if (_this.SelectedAuditorium.MediaBlockType.startsWith('Barco')) {
                url = 'https://' + _this.SelectedAuditorium.MediaBlockIpAddress + ':43744'; ///#sms/player';
            }
            
            var windowObject = window.open(url, "playback-control-" + _this.SelectedAuditoriumNumber);
            _this.mediaBlockTabsAuditorium[_this.SelectedAuditoriumNumber] = windowObject;
        } else {
            _this.mediaBlockTabsAuditorium[_this.SelectedAuditoriumNumber].focus();
        }
    };



    _this.launchTMSPage = function () {
        var url = 'http://' + _this.TmsIpAddress + ':' + _this.TmsPort;
        $window.open(url, "tms");
    };



    var peerConnections = [];
    function createPeerConnection(ipAddress) {
        var newJnrWebsocket = new JnrWebsocket();
        var wsUri = 'ws://' + ipAddress;
        if (window.location.protocol == "https:")
            wsUri = 'wss://' + ipAddress;
        newJnrWebsocket.connect(wsUri);
        newJnrWebsocket.addOnLoggedInListener(function onLoggedIn() {
            console.log('onLoggedIn');

            peerConnections.push(new PeerConnection(newJnrWebsocket, _this.Auditoriums));
        });
    }



    function PeerConnection(jnrWebsocket, auditoriums) {
        var _this = this;
        _this.jnrWebsocket = jnrWebsocket;
        _this.Auditoriums = auditoriums;
        _this.AuditoriumObject = { PeerConnection: this };


        jnrWebsocket.addOnMessageListener(function (jsonReply) {
            var message = jsonReply.Message;
            if ("status.update" === message) {
                /* we received this message.  if the timeout timer exists then cancel it.  this
                also indeicates the first message received. */
                if (timeoutTimer && null != timeoutTimer) {
                    clearTimeout(timeoutTimer);
                    timeoutTimer = null;

                    /* get the auditorium number and add this object to our array */
                    var auditoriumNumber = jsonReply.Data.AuditoriumNumber;
                    if (0 < auditoriumNumber) {
                        _this.AuditoriumObject.AuditoriumNumber = auditoriumNumber;
                        _this.Auditoriums[auditoriumNumber - 1] = _this.AuditoriumObject;

                        _this.jnrWebsocket.readRegistryKey('appdata/ibooth/projectoripaddress', function (keyName, value) {
                            _this.AuditoriumObject.ProjectorIpAddress = value;
                            $scope.$apply();
                        });


                        _this.jnrWebsocket.readRegistryKey('appdata/ibooth/mediablockipaddress', function (keyName, value) {
                            _this.AuditoriumObject.MediaBlockIpAddress = value;
                            $scope.$apply();
                        });
                        _this.jnrWebsocket.readRegistryKey('appdata/ibooth/mediablocktype', function (keyName, value) {
                            _this.AuditoriumObject.MediaBlockType = value;
                            $scope.$apply();
                        });
                        _this.jnrWebsocket.readRegistryKey('appdata/ibooth/cleaninglightchannel', function (keyName, value) {
                            _this.AuditoriumObject.CleaningLightOutput = value;
                            $scope.$apply();
                        });
                    }
                }

                /* update the saved data */
                _this.Auditoriums[_this.AuditoriumObject.AuditoriumNumber - 1].Data = jsonReply.Data;
                $scope.$apply();
            }
        });


        /* we will request an initial status update to be sent.  when we receive a response we 
        will add the auditorium object to the array.  if the timeout expires we will close 
        the connection.  this means that the ibooth application is not running */
        var timeoutTimer = setTimeout(function () {
            _this.jnrWebsocket.close();
        }, 2000);
        _this.jnrWebsocket.postMessage(APP_ID, { Message: 'status.get' });



        _this.controlLights = function (level) {
            _this.jnrWebsocket.postMessage(APP_ID, { Message: 'lights.set', Level: level });
        };


        _this.controlSound = function (level) {
            _this.jnrWebsocket.postMessage(APP_ID, { Message: 'sound.set', Level: level });
        };


        _this.controlProjector = function (level) {
            _this.jnrWebsocket.postMessage(APP_ID, { Message: 'projector.set', Level: level });
        };


        _this.controlMediaBlock = function (command, params) {
            _this.jnrWebsocket.postMessage(APP_ID, { Message: 'mediablock.set', Command: command, Params: params });
        };
    };
});